@@ -1,3 +1,6 @@
+
+/*	$Id: tixGrUtl.c,v 1.2 2004/03/28 02:44:56 hobbs Exp $	*/
+
 /* 
  * tixGrUtl.c --
  *
@@ -23,7 +26,7 @@
 int
 Tix_GetChars(interp, string, doublePtr)
     Tcl_Interp *interp;		/* Use this for error reporting. */
-    char *string;		/* String describing a justification style. */
+    CONST84 char *string;	/* String describing a justification style. */
     double *doublePtr;		/* Place to store converted result. */
 {
     char *end;
@@ -64,9 +67,9 @@ int Tix_GrConfigSize(interp, wPtr, argc, argv, sizePtr, argcErrorMsg,
     Tcl_Interp *interp;
     WidgetPtr wPtr;
     int argc;
-    char **argv;
+    CONST84 char **argv;
     TixGridSize *sizePtr;
-    char * argcErrorMsg;
+    CONST84 char * argcErrorMsg;
     int *changed_ret;
 {
     int pixels;
